# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
generic classes/functions for pyreverse core/extensions
"""

__revision__ = "$Id: lib.py,v 1.2 2003/02/18 16:44:29 syt Exp $"


def time_tag():
    """
    return a timestamp as string
    """
    from time import time, localtime, strftime
    return strftime('%b %d at %T', localtime(time()))

def info(msg):
    print '-'*80
    print msg
    
# PyReverse Visitor methods ###################################################
class BasePyReverseVisitor:
    """
    visitor's methods accoring to the project representation
    """
    def visit_project(self, node):
        pass

    def visit_module(self, node):
        pass
    
    def visit_class(self, node):
        pass
    
    def visit_function(self, node):
        pass
    
    def visit_attribute(self, node):
        pass
    
    def visit_parameter(self, node):
        pass

# Mixins #######################################################################

class IdGeneratorMixIn:
    """
    Mixin adding the ability to generate integer uid
    """
    
    def initCounter(self, start_value=0):
        self.id_count = start_value
        
    def generateId(self):
        """
        generate a new identifer
        """
        self.id_count += 1
        return self.id_count


# PyReverse filter modes #######################################################

_CONSTRUCTOR = 1
_SPECIAL = 2
_PROTECTED = 4
_PRIVATE = 8

FILTERS = {
    'ALL'       : 0,
    'PUB_ONLY'  : _SPECIAL + _CONSTRUCTOR + _PROTECTED + _PRIVATE,
    'SPECIAL'   : _SPECIAL,
    'SPECIAL_A' : _SPECIAL + _CONSTRUCTOR,
    'PRIVATE'   : _PROTECTED + _PRIVATE,
}

class PyReverseFilter:
    
    def __init__(self, mode):
        self._filter_mode = mode
        
    def filter(self, node):
        if self._filter_mode & _PROTECTED and node.visibility == 'protected':
            return 0
        if self._filter_mode & _PRIVATE and node.visibility == 'private':
            return 0
        if self._filter_mode & _CONSTRUCTOR and node.name == '__init__':
            return 0
        if self._filter_mode & _SPECIAL and node.special:
            return 0
        return 1
