# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
utility functions to generate argo uml files
"""

__revision__ = "$Id: argoutils.py,v 1.2 2002/08/30 12:53:13 syt Exp $"

#BASE = 'uci.uml.visual'
BASE = 'org.argouml.uml.diagram.static_structure.ui'
BASE_REL = 'org.argouml.uml.diagram.ui'

from cStringIO import StringIO
def make_argo(modname, version, p_name, files):
    stream = StringIO()

    stream.write('''<?xml version="1.0" encoding="ISO-8859-1" ?>
<!DOCTYPE argo SYSTEM "argo.dtd" >
<argo>
  <documentation>
    <authorname>%s</authorname>
    <version>%s</version>
    <description>
Generated by pyargo tools
    </description>
  </documentation>
  <searchpath href="PROJECT_DIR" />
  <member type="xmi" name="%s.xmi"
  />'''% (modname, version, p_name))

    for file in files:
        stream.write('''
  <member type="pgml" name="%s"/>'''%file)

    stream.write('''
  <historyfile name=""/>
  <stats/>
</argo>''')

    return stream.getvalue()
                 
def open_pgml(type, name, f_uuid):
    return '''<?xml version="1.0" encoding="ISO-8859-1" ?>
<!DOCTYPE pgml SYSTEM "pgml.dtd" >
<pgml description="uci.uml.visual.%s|%s" name="%s">'''%(
        type, f_uuid, name)

def make_FigPackage(f_id, f_uuid):
    return _group("FigPackage", f_id, f_uuid)

def make_FigDependency(new_id, spf, dpf, sfn, dfn):
    return _group('FigDependency', new_id, None, '''
      sourcePortFig="Fig%s.0"
      destPortFig="Fig%s.0"
      sourceFigNode="Fig%s"
      destFigNode="Fig%s"'''%(spf, dpf, sfn, dfn), fill=0, base=BASE_REL)

def make_FigClass(f_id, f_uuid, enclose=''):
    if enclose:
        enclose = 'enclosingFig="%s"' % enclose
    return _group('FigClass', f_id, f_uuid, enclose)

def make_FigInterface(f_id, f_uuid, enclose):
    if enclose:
        enclose = 'enclosingFig="%s"' % enclose
    return _group('FigInterface', f_id, f_uuid, enclose)

def make_FigGeneralization(new_id, spf, dpf, sfn, dfn):
    return _group('FigGeneralization', new_id, None, '''
      sourcePortFig="Fig%s.0"
      destPortFig="Fig%s.0"
      sourceFigNode="Fig%s"
      destFigNode="Fig%s"'''%(spf, dpf, sfn, dfn), fill=1, base=BASE_REL)

def make_FigRealization(new_id, spf, dpf, sfn, dfn):
    return _group('FigRealization', new_id, None, '''
      sourcePortFig="Fig%s.0"
      destPortFig="Fig%s.0"
      sourceFigNode="Fig%s"
      destFigNode="Fig%s"'''%(spf, dpf, sfn, dfn), fill=0, base=BASE_REL)
    

def makeFigAssociation(new_id, spf, dpf, sfn, dfn):
    return _group('FigAssociation', new_id, None, '''
      sourcePortFig="Fig%s.0"
      destPortFig="Fig%s.0"
      sourceFigNode="Fig%s"
      destFigNode="Fig%s"'''%(spf, dpf, sfn, dfn), base=BASE_REL)


def _group(descr, id, idref, private='', fill=1, base=BASE):
    if fill: fill = '''
       fill="1"
       fillcolor="-1"'''
    else: fill = ''
    if idref: idref = '''
       href="%s"''' % idref
    else: idref = ''
    return '''
  <group name="Fig%d"
       description="%s.%s"%s%s
       stroke="1"
       strokecolor="-16777216">
    <private>%s
    </private>
  </group>'''%(id, base, descr, idref, fill, private)

