# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
library to handle diagrams definition
"""

__revision__ = "$Id: diadefslib.py,v 1.2 2003/02/18 16:44:30 syt Exp $"


# saxHandler definition ########################################################
from xmlconf import DictSaxHandlerMixin, PrefReader
import sys

class DiaDefsSaxHandler(DictSaxHandlerMixin):
    """
    definition of the structure of the diadef file, which will enable the MI 
    to fill the DiaDefs dictionary
    """
    _MASTER = {}
    _S_LIST = ('class-diagram', 'package-diagram', 'state-diagram',
               'class', 'package')
    _LIST = ()
    _KEY = ('owner', 'name', 'include')
    _CB = {}


class DiadefsResolverMixIn:
    
    def resolve_packages(self, diadef, list):
        """ take a package diagram definition dictionnary and return matching
        objects
        """
        classes = [] 
        self.resolve_classes(diadef, classes)
        for package in diadef['package']:
            name = package['name']
            c = self._p.get_module(name)
            if c is None:
                print >>sys.stderr, 'Warning: no module named %s'%name
                print >>sys.stderr, c
                continue
            if package.has_key('include'):
                if package['include'] == 'all':
                    for classe in c.children:
                        if classe.TYPE == 'class':
                            classes.append(classe)
                elif package['include'] != 'no':
                    self.resolve_classes(package, classes)
            else:
                self.resolve_classes(package, classes)
            list.append(c)
        return classes

    def resolve_classes(self, diadef, list):
        """ take a class diagram definition dictionnary and return matching
        objects
        """
        for klass in diadef.get('class', []):
            name, module = klass['name'], klass.get('module', '')
            c = self._p.get_classes(name, module)
            if len(c) > 1:
                print >>sys.stderr, 'Warning: more than one class named %s'%name
                print >>sys.stderr, c
                c = c[0]
            elif len(c) == 0:
                print >>sys.stderr, 'Warning: no class named %s'%name
                print >>sys.stderr, c
                continue
            else: c = c[0]
            list.append(c)

def read_diadefs_file(file):
    """ read a diadef file and return the DiaDef dictionnary"""
    diadefs = {} 
    reader = PrefReader(DiaDefsSaxHandler, diadefs)
    try:
        reader.fromFile(file)
    except:
        import traceback
        traceback.print_exc()
        print >>sys.stderr, 'error while reading definitions file %s' % file
    return diadefs

def set_options(optik_parser):
    optik_parser.add_option("-d", "--diadefs", action="store", type='string',
                            dest="diadefs_file", default=None,
                            help="create diagram according to the diagrams definitions in <file>",
                            metavar="<file>")
    
