""" generate a xmi-uml database and an argouml project file
if a diagrams definiton is given (see diadef.dtd), the matching pgml file are
generated
"""
__revision__ = '$Id: pyargo.py,v 1.15 2003/02/26 11:59:24 syt Exp $'

import os
from base.lib import IdGeneratorMixIn, info
from extensions import diadefslib
from extensions.argoutils import *

# argo uml files generator #####################################################
class ArgoWriter(IdGeneratorMixIn, diadefslib.DiadefsResolverMixIn):

    def __init__(self, project, diadef, idstart, uuid_f):
        self.initCounter(idstart)
        self._p = project
        self._d = diadef
        self.f_uuid = uuid_f
        self._files = []
        
    def write(self, xmi):
        self.write_xmi(xmi)
        self.write_pgml()
        self.write_argo()
        
    def write_xmi(self, xmi):
        """
        write to a file the XMI UML definition
        """
        p_f_name = self._p.name.replace(' ', '_')
        info('writing XMI file to %s.xmi' % p_f_name)        
        xmi_file = '%s.xmi' % p_f_name
        xmi_f = open(xmi_file, 'w+')
        from xml.dom.ext import PrettyPrint
        PrettyPrint(xmi, xmi_f)
        xmi_f.close()
        self._files.append(xmi_file)

    def write_pgml(self):
        """
        write necessary pgml files
        """
        diadefs = self._d
        if diadefs:
            for diadef in diadefs.get('class-diagram', []):
                self._write_class_diagram(diadef)
            for diadef in diadefs.get('package-diagram', []):
                self._write_package_diagram(diadef)
            
    def write_argo(self):
        """
        write to a file an argo project description
        
        it must be called once write_xmi and write_pgml have been called
        """
        p_f_name = self._p.name.replace(' ', '_')
        info('writing Argo project file to %s.argo' % p_f_name)
        argo_file = '%s.argo' % p_f_name
        argo_f = open(argo_file, 'w+')
        from logilab.pyreverse import version, modname
        print >>argo_f, make_argo(modname, version, self._p.name, self._files)
        argo_f.close()
        self._files.append(argo_file)

    def zargo(self, zipper='zip'):
        assert self._files
        p_f_name = self._p.name.replace(' ', '_')
        info('zipping all files to %s.zargo' % p_f_name)
        cmd = '%s %s.zargo' % (zipper, p_f_name)
        for f in self._files:
            cmd = '%s %s' % (cmd, f)
        os.system(cmd)
##         import zipfile
##         zip = zipfile.ZipFile('%s.zargo'%p_name, 'w',
##                               zipfile.ZIP_DEFLATED)
##         zip.write(a_file)
##         zip.write(x_file)
##         for f in files:
##             zip.write(f)
##         zip.close()
        
    def clean(self):
        for f in self._files:
            os.remove(f)
        self._files = []

    def _write_package_diagram(self, diadef):
        f = self._open_pgml('UMLClassDiagram',
                            diadef.get('name', 'No name %d'%self.id_count))
        packages = []
        classes = self.resolve_packages(diadef, packages)
        self._write_packages(f, packages)
        self._write_classes(f, classes, packages)
        print >>f, '</pgml>'

    def _write_class_diagram(self, diadef):
        """
        write to a file a class diagram according to the pgml dtd
        """
        f = self._open_pgml('UMLClassDiagram',
                            diadef.get('name', 'No name %d'%self.id_count))
        objects = []
        self.resolve_classes(diadef, objects)
        self._write_classes(f, objects)
        print >>f, '</pgml>'

    def _open_pgml(self, type, name):
        file = '%s.pgml' % name.replace(' ', '_')
        info('creating diagram %s'%file)
        f = open(file, 'w+')
        print >>f,open_pgml(type, name, self.f_uuid(self._p.id))
        self._files.append(file)
        return f

    def _write_packages(self, f, objects):
        for o in objects:
            o._fig_id = self.generateId()
            print >>f, make_FigPackage(o._fig_id, self.f_uuid(o.id))
        # package dependencies
        for o in objects:
            o1 = o._fig_id
            for p in o.get_deps():
                if p in objects:
                    o2 = p._fig_id
                    print >>f, make_FigDependency(self.generateId(), o1, o2, o1, o2)

    def _write_classes(self, f, objects, parents=[]):
        for o in objects:
            m = o.get_module()
            if m in parents:
                enclose = 'Fig%s'%m._fig_id
            else:
                enclose = ''
            m = o.get_module()
            o._fig_id = f_id = self.generateId()
            if o.type == 'interface':
                print >>f, make_FigInterface(f_id, self.f_uuid(o.id), enclose)
            else:
                print >>f, make_FigClass(f_id, self.f_uuid(o.id), enclose)
        links = {}
        # inheritance link
        for o in objects:
            o1 = o._fig_id
            for p in o.parents:
                if p in objects:
                    o2 = p._fig_id
                    new_id = self.generateId()
                    if p.type == 'interface':
                        print >>f, make_FigRealization(new_id, o1, o2, o1, o2)
                    else:
                        print >>f, make_FigGeneralization(new_id, o1, o2, o1, o2)
            # prepare associations
            for attr in o.children:
                if attr.TYPE == 'attribute':
                    try:
                        if attr.default in objects:
                            links.setdefault(o, {})[attr.default] = 1
                            links.setdefault(attr.default, {})[o] = 1
                    except:
                        continue
        # generate associations
        for link, ends in links.items():
            o1 = link._fig_id
            for end in ends.keys():
                o2 = end._fig_id
                print >>f, makeFigAssociation(self.generateId(), o1, o2, o1, o2)
                del links[end][link]
                del links[link][end] 


# main function ################################################################
import sys
    
def Run():
    import config
    for arg in config.get_default_options():
        sys.argv.insert(1, arg)
    from optik import OptionParser
    from base import pyparser
    opt_parser = OptionParser("""USAGE: %prog [options] <package>
parse python sources files and extract an ArgoUML project file from them""")
    # add standard options
    pyparser.set_options(opt_parser)
    diadefslib.set_options(opt_parser)
    # add specific options
    opt_parser.add_option("-z", "--zargo", action="store_true",
                          dest="compress", default=0,
                          help="compress output (ArgoUML > 0.9). Require to have 'zip' in your path or to specify an alternative zip program using the --zipper option")
    opt_parser.add_option("-w", "--zipper", action="store", type='string',
                          dest="zipper", default='zip',
                          help="compress output using <pgm name> to zip all files. Require to have <pgm name> in your path.",
                          metavar="<pgm name>")
    (options, args) = opt_parser.parse_args()
    if not args:
        print opt_parser.print_help()
        return
    
    # extract project representation
    info('parsing project')
    parser = pyparser.PyParser(options.black_list,
                               options.filter_func,
                               options.init_only)
    project = parser.parse_all(args, options.project_name, lambda f,m: sys.stdout.write(
        '  _ parsing file %s\n'%f))
    id_count = parser.id_count
    del parser
    
    # transform into xmi-uml
    from visitors.xmi_uml import XMIUMLVisitor
    info('generating XMI UML')
    xmi_visitor = XMIUMLVisitor(options.filter_func)
    xmi_doc = xmi_visitor.visit(project, id_count)
    id_count = xmi_visitor.id_count
    UUID_format = xmi_visitor.UUID_format
    del xmi_visitor
    
    #  read and interpret diagram definitions
    if options.diadefs_file is not None:
        info('reading diagrams definitions file')
        diadefs = diadefslib.read_diadefs_file(options.diadefs_file)
    # or generate one
    else:
        info('generating diagrams definitions file')
        from visitors.diadefs import DiadefGenerator
        visitor = DiadefGenerator()
        diadefs = visitor.visit(project)

    # write argo uml files
    w = ArgoWriter(project, diadefs, id_count, UUID_format)
    w.write_xmi(xmi_doc)
    del xmi_doc
    w.write_pgml()
    w.write_argo()
    
    # optionaly zip argo project files
    if options.compress:
        w.zargo(options.zipper)
        w.clean()

if __name__ == "__main__":
    Run()

    

