# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""

"""

__revision__ = "$Id: pyvcg.py,v 1.9 2003/02/26 11:59:24 syt Exp $"

import sys

from base.lib import info
from extensions import diadefslib
from logilab.common.vcgutils import VCGPrinter
    
class VCGWriter(diadefslib.DiadefsResolverMixIn):
    def __init__(self, filter, uml):
        self._filter = filter
        self._uml = uml
        
    def write(self, project, diadefs):
        self._p = project
        self._files = []
        if diadefs:
            for diadef in diadefs.get('class-diagram', []):
                self.write_class_diagram(diadef)
            for diadef in diadefs.get('package-diagram', []):
                self.write_package_diagram(diadef)

    def write_package_diagram(self, diadef):
        print 'package diagram not yet implemented'
        return
        self._open_vcg(diadef.get('name', 'No name package diagram'))
        packages = []
        classes = self.resolve_packages(diadef, packages)
        self._write_packages(packages)
        self._write_classes(classes, packages)
        self.printer.close_graph()

    def write_class_diagram(self, diadef):
        """ write to a file a class diagram according to vcg syntax
        """
        f = self._open_vcg(diadef.get('name', 'No name class diagram'))
        objects = []
        self.resolve_classes(diadef, objects)
        self._write_classes(objects)
        self.printer.close_graph()
        f.close()

    def _open_vcg(self, name):
        name = name.strip()
        file = ('%s.vcg' % name.replace(' ', '_'))
        info('creating diagram %s'%file)
        f = open(file, 'w+')
        self._files.append(file)
        self.printer = VCGPrinter(f)        
        self.printer.open_graph(title=name, 
                                layoutalgorithm='dfs',
                                late_edge_labels='yes',
                                port_sharing='no',
                                manhattan_edges='yes'
                                )
        return f
    
    def _write_packages(self, objects):
        for o in objects:
            o._fig_id = self.generateId()
            print >>f, make_FigPackage(o._fig_id, self.f_uuid(o.id))
        # package dependencies
        for o in objects:
            o1 = o._fig_id
            for p in o.get_deps():
                if p in objects:
                    o2 = p._fig_id
                    print >>f, make_FigDependency(self.generateId(), o1, o2, o1, o2)

    def _write_classes(self, objects, parents=[]):
        for o in objects:
##             m = o.get_module()
##             if m in parents:
##                 enclose = 'Fig%s'%m._fig_id
##             else:
##                 enclose = ''
##             o._fig_id = f_id = self.generateId()
##             if o.type == 'interface':
            if o.signal == 'true':
                label = r'\fb\f09%s\fn' % o.name
            else:
                label = r'\fb%s\fn' % o.name
            if o.type == 'interface':
                shape = 'ellipse'
            else:
                shape = 'box'
            delayed = []
            if self._uml:
                label = r'%s\n\f05_______________' % label
            for attr in o.children:
                if self._filter(attr):
                    if attr.TYPE == 'attribute':
                        label = r'%s\n\f08%s' % (label, attr.name)
                    else:
                        delayed.append(attr)
            if self._uml:
                label = r'%s\n\f05_______________' % label
            for func in delayed:
                label = r'%s\n\f10%s()' % (label, func.name)
                
            self.printer.node(o.name, label=label, shape=shape)
#             else:
#                 self.printer.node(o.name)
        links = {}
        # inheritance link
        for o in objects:
            for p in o.parents:
                if p in objects:
                    # FIXME: pb with backarrow
                    if p.type == 'interface':
                        self.printer.edge(o.name, p.name, linestyle='dotted',
                                          arrowstyle='line')
                    else:
                        self.printer.edge(o.name, p.name, arrowstyle='solid')
            # prepare associations
            for attr in o.children:
                if attr.TYPE == 'attribute':
                    try:
                        if attr.default in objects:
                            links.setdefault(o, {})[attr.default] = 1
                            links.setdefault(attr.default, {})[o] = 1
                    except:
                        continue
        # generate associations
        for link, ends in links.items():
            for end in ends.keys():
                self.printer.edge(link.name, end.name)
                del links[end][link]
                del links[link][end] 

# main function ################################################################

def Run():
    import config
    for arg in config.get_default_options():
        sys.argv.insert(1, arg)
    from optik import OptionParser
    from base import pyparser
    opt_parser = OptionParser("""USAGE: %prog [options]
parse python sources files and extract a class diagram readable by vcg""")
    # add standard options
    pyparser.set_options(opt_parser)
    diadefslib.set_options(opt_parser)
    # add specific options
    opt_parser.add_option("-u", "--uml", action="store_true",
                          dest="uml", default=0,
                          help="make diagram looks like an UML diagram")
    (options, args) = opt_parser.parse_args()
    if not args:
        print opt_parser.print_help()
        return

    # extract project representation
    info('parsing project')
    parser = pyparser.PyParser(options.black_list,
                               options.filter_func,
                               options.init_only)
    project = parser.parse_all(args, p_name, lambda f,m: sys.stdout.write(
        '  _ parsing file %s\n'%f))
    id_count = parser.id_count
    del parser
    #  read and interpret diagram definitions
    if diadefs_file is not None:
        diadefs = diadefslib.read_diadefs_file(options.diadefs_file)
    # or generate one
    else:
        from visitors.diadefs import DiadefGenerator
        visitor = DiadefGenerator()
        diadefs = visitor.visit(project)
    w = VCGWriter(options.filter_func, options.uml)
    w.write(project, diadefs)

if __name__ == "__main__":
    Run()
