# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
"""

__revision__ = "$Id: diadefs.py,v 1.3 2003/02/18 10:57:09 syt Exp $"

from logilab.common.visitor import Visitor
from logilab.common.tree import PrefixedDepthFirstIterator

def diadef_filter(node):
    # take only first level classes
    if node.TYPE == 'class' and node.parent.TYPE != 'module':
        return 0
    # don't care about functions and attributes
    if node.TYPE == 'function' or node.TYPE == 'attribute':
        return 0
    return 1

class DiadefGenerator(Visitor):
    """
    generate minimum diagram definition for the project

    _S_LIST = ('class-diagram', 'package-diagram', 'state-diagram',
               'class', 'package')
    _LIST = ()
    _KEY = ('owner', 'name', 'include')
    """

    def __init__(self):
        Visitor.__init__(self, PrefixedDepthFirstIterator, diadef_filter)

    def open_visit(self, node):
        self._diadefs = {}
        
    def close_visit(self, result):
        return self._diadefs

    def visit_project(self, node):
        diadefs = self._diadefs
        if len(node.children) > 1:
            diadefs['package-diagram'] = [{'name' : 'packages %s'%node.name,
                                           'package': []}]
        diadefs['class-diagram'] = [{'name' : 'classes %s'%node.name,
                                     'class': []}]
    
    def visit_module(self, node):
        if len(node.parent.children) > 1:
            self._diadefs['package-diagram'][0]['package'].append({'name' : node.name})
    
    def visit_class(self, node):
        self._diadefs['class-diagram'][0]['class'].append(
            {'name' : node.name, 'owner': node.parent.name})

    def visit_function(self, node):
        # don't care about function
        return
    
    def visit_attribute(self, node):
        # don't care about attribute
        return
