# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
"""

__revision__ = "$Id: test_generator.py,v 1.2 2003/02/18 10:11:15 syt Exp $"

from logilab.common.visitor import Visitor
from logilab.common.tree import PrefixedDepthFirstIterator
from logilab.pyreverse.extensions.testutils import *
from logilab.pyreverse.base import objects

class UnittestGeneratorVisitor(Visitor):
    
    def __init__(self, generated_dir='tests', header='', filter=None):
        Visitor.__init__(self, PrefixedDepthFirstIterator)
        self._filter = filter
        self.filter = self.test_filter
        #FilteredVisitor.__init__(self)
        self.tests_dir = generated_dir
        self.header = header
        self._stream = None
        
    def test_filter(self, node):
        if self._filter is not None and not self._filter(node):
            return 0
        # take only first level classes which are not interface nor signal
        if node.TYPE == 'class' and (node.parent.TYPE != 'module'
                                     or node.type == 'interface' 
                                     or node.signal == 'true'):
            return 0
        if  node.TYPE == 'function' and node.name == '__init__':
            return 0    
        return 1

    def close_visit(self, result):
        self._close_test()
        
    def visit_project(self, node):
        pass
    
    def visit_module(self, node):
        import os, sys
        from os.path import join, exists
        if self._stream is not None:
            self._close_test()
        file = join(self.tests_dir, 'unittest_%s.py'%node.name.replace('.', '_'))
        if exists(file):
            os.rename(file, '%s~'%file)
        self._stream = open(file, 'w+')
        os.chmod(file, 0777)
        self._stream.write(open_unittest(sys.argv[0], node.name, self.header,
                                         doc=node.doc))
        self._tests = []
        self._test_class = None
    
    def visit_class(self, node):
        self._test_class = test = '%s_ClassTest' % node.name
        self._stream.write(open_testcase(
            test, node.doc,
            'self.o = %s(%s)'%(node.name, get_class_init_args(node))))
        
    def visit_function(self, node):
        if node.parent.TYPE != 'class':
            self._test_class = test = '%s_FunctionTest' % node.name
            self._stream.write(open_testcase(test))
            object = ''
        else:
            object = 'self.o.'
        test_name = _name('knownValues', node.name, '1')
        self._tests.append((self._test_class, test_name))
        self._stream.write(open_func(test_name))
        self._stream.write(whatever('"""%s"""'%node.doc))
        self._stream.write(assert_equal(
            '%s%s(%s)'%(object, node.name, get_function_init_args(node)),
            get_function_result(node)))
        for exception in node.raises:
            try:
                exception = exception.name
            except AttributeError:
                pass
            test_name = _name('raise', node.name, exception.split('(')[0])
            self._tests.append((self._test_class, test_name))
            self._stream.write(open_func(test_name))
            self._stream.write(assert_raise(exception, 'self.o.%s, %s' % (
                node.name, get_function_raise_args(node, exception))))

    def visit_attribute(self, node):
        pass
    
    def visit_parameter(self, node):
        pass
    
    def _close_test(self):
        self._stream.write(close_unittest(self._tests))
        self._stream.close()
        self._stream = None

def _name(group,name, tid):
    return '%s_%s_%s'%(group,name, tid)

def get_class_init_args(node):
    for child in node.children:
        if child.name == '__init__':
            return get_function_init_args(child)
    return ''

def get_function_init_args(node):
    res = ''
    for param in node.children:
        res = '%s, %s' % (res, param.name)
        try:
            res = '%s=%s' % (res, param.default)
        except:
            pass
    return res and res[2:] or res

def get_function_result(node):
    return ''
    
def get_function_raise_args(node, exception):
    return ''
