# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
convert the class hierarchy describing the project to XMI 1.0 UML 1.3
DOM representation

please notices that it is based on file generated by ArgoUML
"""

__revision__ = "$Id: xmi_uml.py,v 1.5 2003/02/18 10:57:09 syt Exp $"

import sys
try:
    # try to use 4Suite cDomlette if available
    from Ft.Xml.cDomlette import implementation
    print >>sys.stderr, ':: using cDomlette'

    def Document(namespace, q_name, pub_id, sys_id):
        doctype = implementation.createDocumentType(q_name, pub_id, sys_id)
        doc = implementation.createDocument(namespace, q_name, doctype)
        return doc
     
except:
    # 
    from xml.dom.minidom import DOMImplementation
    implementation = DOMImplementation()
    print >>sys.stderr, ':: using minidom'
    
    def Document(namespace, q_name, pub_id, sys_id):
        doctype = implementation.createDocumentType(q_name, pub_id, sys_id)
        doc = implementation.createDocument(namespace, q_name, doctype)
        root = doc.documentElement
        doc.appendChild(root)
        return doc
    
from logilab.common.visitor import Visitor
from logilab.common.tree import PrefixedDepthFirstIterator
from logilab.pyreverse.extensions.xmiutils import *
from logilab.pyreverse.base.objects import Klass, AbstractBase
from logilab.pyreverse.base.lib import IdGeneratorMixIn

# FIXME: shouldn't use XMI_* constantes here (13 more)

MAX_CHAR = 20


class XMIUMLVisitor(IdGeneratorMixIn, Visitor):
    """
    generate a dom tree conform to the XMI-1.0 UML-1.3 DTD from the project
    description
    """
    def __init__(self, filter=None):
        Visitor.__init__(self, PrefixedDepthFirstIterator)
        self._filter = filter
        self.filter = self.uml_filter
        
    def open_visit(self, node, idstart, doc_factory=Document):
        """
        initialize the XMI-UML visitor
        """
        self._uuid = '127-0-0-1-unbelid:entifiant:'
        self.initCounter(idstart)
        self.doc_factory = doc_factory
        self._xref_tab = {}
        self._signals = {}
        self._signal_objects = {}
        self._modules = {}
        self._attributes = []
        
    def uml_filter(self, node):
        if self._filter is not None and not self._filter(node):
            return 0
        # take only first level classes
        if node.TYPE == 'class' and node.parent.TYPE != 'module':
            return 0
        # take only classes methods
        if node.TYPE == 'function' and node.parent.TYPE != 'class':
            return 0
        # take only classes/instances attributes
        if node.TYPE == 'attribute' and node.parent.TYPE != 'class':
            return 0
        return 1


    def close_visit(self, result):
        """
        do some post processing when all nodes have been visited
        """
        doc = self._doc
        # resolve modules dependences
        delayed = []
        for module, dom_n in self._modules.items():
            deps = module.get_deps()
            for dep in deps:
                n_id = self.generateId()
                d = xmi_dependency(doc, n_id, self.UUID_format(n_id),
                                   module.id, dep.id, self._project)
                before = dom_n.lastChild
                while before.previousSibling.nodeName == XMI_MODEL_ELMT_SD:
                    before = before.previousSibling
                d = add_me_dependency(dom_n, client=1, before=before)
                add_idref(d, XMI_DEPENDENCY, n_id)
                delayed.append
                d = add_me_dependency(self._modules[dep],
                                      before=self._modules[dep].lastChild,
                                      client=0)
                add_idref(d, XMI_DEPENDENCY, n_id)
        # process generalization
        for c, p in self._xref_tab.values():
            p.appendChild(c)
        # process exceptions
        for parent, children in self._signals.items():
            for child in children:
                dom_node, content = self._signal_objects[child] #self._open_class_def(child)
                if hasattr(child, 'context'):
                    for c in child.context:
                        me = doc.createElementNS(NO_NS, XMI_BE_SIGNAL_CONT)
                        dom_node.appendChild(me)
                        add_idref(me, XMI_BF, c.id)
                        parent.appendChild(dom_node)
        # process associations
        for child in self._attributes:
            dom_node = self._open_association(child)
            self._project.appendChild(dom_node)
        return doc
    

    def visit_project(self, node):
        """ initialize an xmi dom tree """
        doc = self._doc = xmi_document(self.doc_factory)
        # add UUID format function to the document for convenience
        #doc.UUID_format = self.UUID_format
        self._project = xmi_content(doc, node.id, self.UUID_format(node.id),
                                    node.name)

    def visit_module(self, node):
        package, content = xmi_package(self._doc, node.id,
                                       self.UUID_format(node.id),
                                       node.name, node.visibility)
        self._project.appendChild(package)
        self._signals[content] = []
        self._modules[node] = package
        # push modules
        self._module = content
    
    def visit_class(self, node):
        n, content = self._open_class_def(node)
        if node.signal == 'true':
            # delay attachment to be conform to the dtd
            self._signals[self._module].append(node)
            self._signal_objects[node] = (n, content)
        else:
            self._module.appendChild(n)
        # push classes
        self._class = content
    
    def visit_function(self, node):
        doc = self._doc
        function = xmi_operation(doc, node.id,  self.UUID_format(node.id),
                                 node.name, node.visibility,
                                 node.final, node.type)
        self._class.appendChild(function)
        if node.children:
            content = doc.createElementNS(NO_NS, XMI_BF_PARAM)
            function.appendChild(content)
        else:
            content = None
        # push functions
        self._function = content
    
    def visit_attribute(self, node):
        default = None
        if hasattr(node, 'default'):
            # is the default value of this attribute a link to a known instance
            if isinstance(node.default, Klass):
                self._attributes.append(node)
            # or a string ?
            elif not isinstance(node.default, AbstractBase):
                default = node.default[:MAX_CHAR]
        n_id = self.generateId()
        attr = xmi_attribute(self._doc, self.generateId, n_id,
                             self.UUID_format(n_id),
                             node.name, node.visibility, default)
        self._class.appendChild(attr)
        
    def visit_parameter(self, node):
        self._function.appendChild(xmi_parameter(
            self._doc, node.id, node.name, node.visibility))

    # protected methods ########################################################
    
    def _open_class_def(self, node):
        doc = self._doc
        interface = (node.type == 'interface')
        if interface:
            el = xmi_interface(doc, node.id, self.UUID_format(node.id),
                               node.name, node.visibility,
                               node.final, node.abstract)
        elif node.signal == 'true':
            el = xmi_signal(doc, node.id, self.UUID_format(node.id),
                            node.name, node.visibility,
                            node.final, node.abstract)
        else:
            el = xmi_class(doc, node.id, self.UUID_format(node.id),
                           node.name, node.visibility,
                           node.final, node.abstract)
        spec, klass = None, []
        # are there specializations of this classes
        if node.subclasses:
            if interface:
                pe = add_me_dependency(el, client=0)
                a_name = XMI_ABSTRACTION
            else:
                # delay attachment to be conform to the dtd
                spec = pe = doc.createElementNS(NO_NS, XMI_G_ELMT_SPEC)
                a_name = XMI_GENERALIZATION
            for p in node.subclasses:
                if isinstance(p, Klass):
                    add_idref(pe, a_name, self._get_x_ref(node, interface))
        # are we a specialization of some classes
        for p in node.parents:
            if isinstance(p, Klass):
                if p.type == 'interface':
                    pe = add_me_dependency(el, client=1)
                    add_idref(pe, XMI_ABSTRACTION, self._get_x_ref(p, 1, node))
                else:
                    klass.append(p)
        if klass:
            pe = doc.createElementNS(NO_NS, XMI_G_ELMT_GENE)
            el.appendChild(pe)
            for p in klass:
                add_idref(pe, XMI_GENERALIZATION, self._get_x_ref(p, 0, node))

        # is there a delayed node to attach ?
        if spec is not None:
            el.appendChild(spec)
        # do we need a children container ?
        if node.children:
            content = doc.createElementNS(NO_NS, XMI_FEATURE)            
            el.appendChild(content)
        else:
            content = None
        return el, content
    

    def _open_association(self, attr_n):
        if hasattr(attr_n, '_resolved_'):
            return
        attr_n._resolved_ =1
        for c in attr_n.default.children:
            if c.TYPE == 'attribute' and hasattr(c, 'default') and c.default == attr_n:
                c._resolved_ = 1
                nav = 'true'
                break
        else:
            nav = 'false'
        a = []
        for attr in (attr_n.get_class(), attr_n.default.get_class()):
            if attr.type == 'interface':
                a.append(('interface', attr.id))
            elif attr.signal:
                a.append(('signal', attr.id))
            else:
                a.append(('class', attr.id))
        n_id = self.generateId()
        return xmi_association(self._doc, self.generateId,
                               n_id, self.UUID_format(n_id),
                               attr_n.name, a[0][0], a[0][1],
                               attr_n.default.name, a[1][0], a[1][1], nav)


    def _get_x_ref(self, root, interface=None, dependency=None):
        """ manage cross links between two objects (generalization, implements)
        root is the ancestor node.
        This function is called two time for each link:
          _ one with dependency=None
          _ one with dependency references the specialization object.
        We don't know the order :(
        """
        doc = self._doc
        if not hasattr(root, '_tmp_xmi_ref'):
            root._xmi_ref = {}
            root._last_xmi_ref, root._tmp_xmi_ref = [], []
        if dependency is not None and root._tmp_xmi_ref:
            root._xmi_ref[dependency] = e = root._tmp_xmi_ref[-1]
            xmi_X_ref(doc, self._xref_tab[e][0], interface, None, dependency.id)
            del root._tmp_xmi_ref[-1]
            return e
        elif dependency is None and root._last_xmi_ref:
            e = root._xmi_ref[root._last_xmi_ref[-1]]
            xmi_X_ref(doc, self._xref_tab[e][0], interface, root.id, None)
            del root._last_xmi_ref[-1]
            return e
        # create dependency
        base_id = self.generateId()
        if dependency:
            root._xmi_ref[dependency] = base_id
            root._last_xmi_ref.append(dependency)
        else: 
            root._tmp_xmi_ref.append(base_id)
        if interface:
            xref = xmi_abstraction(doc, base_id, self.UUID_format(base_id))
            # extension
            ext_id = self.generateId()
            ext = xmi_stereotype(doc, base_id, ext_id)
        else:
            xref = xmi_generalization(doc, base_id, self.UUID_format(base_id))
            ext = None
        if ext is not None:
            self._xref_tab[ext_id] = (ext, self._module) 
            xref.appendChild(xmi_stereotype_ref(doc, ext_id))
        xmi_X_ref(doc, xref, interface,
                  root and root.id, dependency and dependency.id)
        self._xref_tab[base_id] = (xref, self._module)
        return base_id
    

    def UUID_format(self, id):
        """
        format XMI UUID
        """
        return '%s-%x' % (self._uuid, 0x8000 - id)
